#!/usr/bin/perl

##################################################
#
# GiantDisc mp3 Jukebox
# 
#  2000, Rolf Brugger
#
##################################################


# gdplayd.pl: Giant Disc play playlist deamon
#   When started, reads the current tracks form the playlist file and plays 
#   them sequentally.
#   Actions: (This script is only started, if something has to be played/mp3-decoded.
#             If playing is stopped or paused, this script should be terminated)
#     1) Reads the playlist and playstate table
#     2) Play tracks starting from the 'current' track
#     3) if the 'current' track comes to its end, the next track is played
#        and the playstate table is updated
#     4) while playing a track, playstate table containing the current 
#        playing position is permanently updated
#
#   The script terminates, when all tracks are played 

use lib '/home/music/bin';
use gddb;
use gdparams;
use gdgentools;

use DBI;



### constants
my $soundcard0   = 0;   # main sound output of first soundcard
my $soundcard1   = 1;   # alternative sound output on second s9undcard (not supported) 
my $mp3streamer  = 20;  # mp3 streamer (barix exstreamer)



### Parameters
if(scalar(@ARGV) != 3){print("gdplayd needs three parameters (host-ip, id and nb)\n");exit(0);}
my $dbhost   = $ARGV[0];
my $playerid = $ARGV[1];
my $audiochannel = $ARGV[2]; 

### open DB connection
my $dbh = DBI->connect("DBI:mysql:GiantDisc:".$dbhost, "music", undef)
    or die "unable to connect to GiantDisc db";

### set termination handler
$SIG{TERM} = sub { print "playd killed\n"; $dbh->disconnect; exit;}; # set Kill event handler


### write my PID to DB, that I can be killed efficiently
#   -> unstable and too slow when killed with Proc::Killall or Proc::Killfam
#print "my pid is \"".$$."\"\n";
gdgentools::pll_write_player_processid($dbh, $playerid, $audiochannel, $$);


### Begin
my $base = gdparams::gdbase();
my ($snddevice, $playerapp, $playerparams, $ptlogger, $shufflepar) =
	gdgentools::pll_get_playparams($dbh, $playerid, $audiochannel);

my ($mp3playerparams, $oggplayerparams) = split /\t/, $playerparams;

### define play time filter pipe
my ($mp3filterpipe, $oggfilterpipe);
if ($ptlogger =~ /\/dev\/null/){
  # no filter pipe
  $mp3filterpipe = "> /dev/null";
  $oggfilterpipe = "> /dev/null";
}
else{
  $mp3filterpipe = "| $ptlogger $dbhost $playerid $audiochannel mp3";
  $oggfilterpipe = "| $ptlogger $dbhost $playerid $audiochannel ogg";
  $flacfilterpipe= "| gdplaytmsim.pl $dbhost $playerid $audiochannel";
}


my ($sth, $row, $audiofile, $trackind, $trackid, $state, $startframe);
my ($fileformat, $dummy, $audioparam, $startpos, $startsamp);
my ($playertype, $playerhost, $sounddevice);

### Get player type
($playertype, $playerhost, $sounddevice) = gdgentools::pll_get_playertype($dbh, $playerid, $audiochannel);

### Get State of player
($trackind, $state, $startframe, $dummy) = gdgentools::pll_get_playstate($dbh, $playerid, $audiochannel);

if (length($shufflepar) > 1){ # are we in shuffle play mode?
  gdgentools::tracklist_del_upto_item($dbh, $playerid, $audiochannel, $trackind);
  ### generate next tracks to be played
  fill_random_tracks($dbh, $playerid, 0, $shufflepar);
  $trackind = 0;
}

# get first track to be played
$trackid = gdgentools::tracklist_get_item($dbh, $playerid, 0, $trackind);

###
#print "gdplayd.pl sends message now ($playerid, $audiochannel)\n";
gdgentools::plmsg_send($playerid, $audiochannel);
#print "gdplayd.pl: message sent\n";

while($trackid){
  gdgentools::pll_write_playstate($dbh, $playerid, $audiochannel, $trackind, "pl");

  $sth = $dbh->prepare("SELECT * FROM tracks WHERE id = $trackid");
  $sth->execute;
  $row = $sth->fetchrow_hashref;
  $sth->finish;

  ($fileformat, $audioparam) = gdgentools::bitrate_str_to_format_param($row->{bitrate});  # split "mp3 128"
  $audiofile = "";
  if (gdgentools::is_mp3stream($row->{mp3file})){
    ### it's a URL -> stream it
    $audiofile = $row->{mp3file};
  }
  else{
    ### it's a normal audio file -> retrieve it in one of the audio file directories
    if (length($row->{mp3file})>4){
      $audiofile = `ls $base/[0-9][0-9]/$row->{mp3file}`;
      chop ($audiofile);
    }else{print "Error: audiofile'".$row->{mp3file}."' broken?\n"}
  }
  if(length($audiofile)>4){
    $startsec  = int($startframe / gdgentools::frames_per_second()); # frames->sec
    $startsamp = int($startframe * gdgentools::samples_per_frame());

    print("playing $audiofile ($row->{title}) starting at $startsec sec, format: '$fileformat'\n");
    ### while playing, redirect std-err to playtime logger

    ####################################################################################
    ### play on soundcard
    if ($playertype == $soundcard0){
      if ($fileformat eq "mp3"){     ### mp3 ###
        #print( "mpg123 -v $mp3playerparams --skip $startframe --aggressive $audiofile 2>&1 $mp3filterpipe\n");
        system( "mpg123 -v $mp3playerparams --skip $startframe --aggressive "
               ."$audiofile 2>&1 $mp3filterpipe");
      }
  
      elsif ($fileformat eq "ogg"){  ### ogg ###
        #print( "ogg123 -v $oggplayerparams --skip $startsec $audiofile 2>&1 $oggfilterpipe \n");
        system( "ogg123 -v $oggplayerparams --skip $startsec $audiofile 2>&1 $oggfilterpipe");
      }
  
      elsif ($fileformat eq "flac"){ ### flac ###
        ### we have to write the total track length (used by playtimesimulator)
        gdgentools::pll_write_playtime($dbh, $playerid, $audiochannel, 
              $startframe, int($row->{length} * gdgentools::frames_per_second()) );
        #print ("flac -c -d -s --skip $startsamp --endian=little --sign=signed --force-raw-format $audiofile | rawplay -f s16_le $flacfilterpipe \n");
        system ("flac -c -d -s --skip $startsamp --endian=little --sign=signed --force-raw-format $audiofile | rawplay -f s16_le $flacfilterpipe");
      }
  
      else{
        print "Player Error: unknown file type $fileformat of $audiofile\n";
      }
    }
    
    ####################################################################################
    ### send audio stream
    elsif($playertype == $mp3streamer){
      print  ("gdstream.pl -h $playerhost -p $sounddevice -b ".$audioparam*1000
             ." -f $fileformat -s $startsec -l ". $row->{length}
             ." -d $dbhost -i $playerid -c $audiochannel $audiofile\n");
      system ("gdstream.pl -h $playerhost -p $sounddevice -b ".$audioparam*1000
             ." -f $fileformat -s $startsec -l ". $row->{length}
             ." -d $dbhost -i $playerid -c $audiochannel $audiofile");
      sleep 1;    # give input buffer time to recover
    }

    else{
        print "Player Error: unknown player type $playertype\n";
    }
    
    $startframe = 0; # play next tracks from beginning
  }
  else {
    print("ERROR: could not find mp3 file! ID=$row->{sourceid}, track=$row->{tracknb}\n");
  }  

  if (length($shufflepar) > 1){ # are we in shuffle play mode?
    ### generate next tracks to be played; trackindex remains 0!
    gdgentools::tracklist_del_item($dbh, $playerid, 0, 0); # remove 0th item
    fill_random_tracks($dbh, $playerid, 0, $shufflepar);
  }
  else{                         # are we in normal play mode
    ### go to next track
    $trackind++;
  }
  ### get track-ID at current index
  $trackid = gdgentools::tracklist_get_item($dbh, $playerid, 0, $trackind);
}
gdgentools::pll_write_playstate($dbh, $playerid, $audiochannel, $trackind, "st");
gdgentools::pll_write_playtime($dbh, $playerid, $audiochannel, 
        0,   # frames played
        0);  # frames total


### clear my PID in DB, telling that player is inactive
#gdgentools::pll_write_player_processid($dbh, $playerid, $audiochannel, $$);

$dbh->disconnect;
exit(0);


####################################################################################

### Shuffle play functions
sub fill_random_tracks{
  my ($dbh, $playerid, $listtype, $shufflepar) = @_;
  my $tracks_predetermined = 5;
  #print("we have".gdgentools::tracklist_get_nb_items($dbh, $playerid, $listtype)." tracks\n");
  while(gdgentools::tracklist_get_nb_items($dbh, $playerid, $listtype) < $tracks_predetermined){

    my $where = gddb::track_where_clause($gddb::get_tracks, split('\t', $shufflepar));
    #print("whereclause: $where\n");
    my ($row, $trackid, $sth);
    $sth = $dbh->prepare( 
         "SELECT id,(id*0)+RAND() AS randnum FROM tracks "
        ."WHERE $where ORDER BY randnum LIMIT 1" );
    $sth->execute;
    $trackid = 1; # in case of an error we want to have something
    if($row = $sth->fetchrow_hashref){
      $trackid = $row->{id};
    }
    #print("adding random track id=$trackid\n");
    gdgentools::tracklist_append_list($dbh, $playerid, $listtype, $trackid);
    $sth->finish;
  } # end while
}
#

####################################################################################
