/* 
   C-Version of gdplaytimefilter.pl
   gdplaytimefilter: Giant Disc play time filter
   reads play time state from mpg123 (stdin) and writes it to the corresponding
   playerstate-record in the database.


   we need to read the mpg123 status char by char. \r must be interpreted
   as line end (translation \r -> \n in a pipe with 'tr' is not possible,
   because tr writes buffered to stdout. But we need undelayed response.)
   Better ideas?


   The program takes the following parameters
     1) int playerid	 player-ID
     2) int audiochannel sound out of player-id
     3) char *gdhost	 IP address or hostname of database host


                         44100 samples/sec
   38.281 frames/sec =   ------------------
                         1152 samples/frame
 */

#include <stdio.h>
#include <stdlib.h>
//#include <unistd.h>
//#include <sys/types.h>

#include "mysql/mysql.h"

#define ftype_mp3   0
#define ftype_ogg   1
#define forthsec    10      /*nb frames in a forth of a second (roughly)*/

main (int argc, char **argv)
{
  char line[500], sqlcmd[500], *pos2, *pos3, *playerid, *audiochannel, *gdhost;
  int  chr, chrcnt, linelen, lineok, frames, framestotal, frameslastupdate;
  int  ftype;
  int  min, sec, hsec;	// ogg time calculations
  MYSQL mysql;

  if(argc != 5){
    printf("gdplaytimefilter needs 4 parameters: playerid, audiochannel, gdhost, filetype (mp3, ogg, ...) \n");
    exit (-1);
  }
  gdhost   = argv[1];
  playerid = argv[2];
  audiochannel = argv[3];
  if (strcmp(argv[4], "mp3")==0) ftype = ftype_mp3;
  if (strcmp(argv[4], "ogg")==0) ftype = ftype_ogg;
  
  //printf ("Play Time Filter started. params %s %s %s\n", playerid, audiochannel, gdhost);

  //*** Open Database connection
  mysql_init(&mysql);
  if(mysql_real_connect(&mysql, gdhost, NULL/*user*/, NULL/*pwd*/,
  	"GiantDisc", 0/*port*/, NULL/*socket*/, 0/*client-flag*/))
  {
    //*** Init
    sprintf(sqlcmd,
      "UPDATE playerstate \
        SET framesplayed=%i, framestotal=%i \
        WHERE playerid=%s AND audiochannel=%s",
        0/*frames*/, 0/*framestotal*/, playerid, audiochannel
      );
    mysql_query(&mysql, sqlcmd);
    frameslastupdate = 0;
    
    //*** Get mpg123 verbose output
    //*   Example of an output line:
    //*   Frame#   210 [14226], Time: 00:05.48 [06:11.61],


    //*** Get ogg123 verbose output
    //*   Example of an output line:
    //*   Track number: 3
    //*   Album: the revolution
    //*   Genre: 43
    //*   Time: 00:03.78 [05:36.04] of 05:39.83  ( 96.9 kbps)  Output Buffer  96.9%
    //*   Time: 00:04.06 [05:35.76] of 05:39.83  (104.9 kbps)  Output Buffer  96.9%

    while(1){
      linelen = 0;
      lineok  = 0;
      chrcnt  = 0;
      if (ftype == ftype_mp3){
        while ((chr=getchar()) != EOF){
          if (chrcnt==0 && chr == 'F') lineok=1;  // lines start with keyw "Frame#"
          if (chr == '\r' || chr == '\n') break;
          if (isdigit(chr) || chr == ' '){
        	line[linelen++] = chr;
          }
          chrcnt++;
        }
      }
      
      if (ftype == ftype_ogg){
        while ((chr=getchar()) != EOF){
          if (chrcnt==0 && chr == 'T') lineok=1;  // lines start with keyw "Time:"
          if (chrcnt==1 && chr != 'i') lineok=0;
          if (chrcnt==2 && chr != 'm') lineok=0;
          if (chr == '\r' || chr == '\n') break;
          if (isdigit(chr)){ line[linelen++] = chr;}
          else{              line[linelen++] = ' ';}
          chrcnt++;
        }
      }
      
      line[linelen++] = '\0';
      if (chr == EOF) break;
  
      frames = 0;
      framestotal = 0;
      if (lineok){
        //printf("line recieved: %s \n", line);
        if (ftype == ftype_mp3){
          frames      = strtol(line, &pos2, 10); // pos2 points to first char after first number
          framestotal = strtol(pos2, &pos3, 10)/*frames-ramining*/+frames;
        }
        if (ftype == ftype_ogg){
          min  = strtol(line, &pos2, 10); 
          sec  = strtol(pos2, &pos3, 10); 
          hsec = strtol(pos3, &pos2, 10); 
          frames = (((min*6000)+(sec*100)+hsec) * 441) / 1152;

          min  = strtol(pos2, &pos3, 10); // skip 3 numbers
          min  = strtol(pos3, &pos2, 10);
          min  = strtol(pos2, &pos3, 10);

          min  = strtol(pos3, &pos2, 10); 
          sec  = strtol(pos2, &pos3, 10); 
          hsec = strtol(pos3, &pos2, 10); 
          framestotal = (((min*6000)+(sec*100)+hsec) * 441) / 1152;
        }
        
        if (frames > (frameslastupdate+forthsec)){ // not more than 4 updates/second
          sprintf(sqlcmd,
    	     "UPDATE playerstate SET framesplayed=%i, framestotal=%i \
              WHERE playerid=%s AND audiochannel=%s",
              frames, framestotal, playerid, audiochannel
    	    );
          //printf("QUERY: %s\n", sqlcmd);
          mysql_query(&mysql, sqlcmd);
          frameslastupdate = frames;
        }
      }
    }
  }
  else{
    printf("ERROR: gdplaytimefilter can't open database connection\n");
    printf("       %s\n", mysql_error(&mysql));
  }
  //printf ("Terminated: Playtime filter\n");
}

